#!/usr/bin/perl -w

use CGI qw/:standard *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e) = @_;
    "<img src=\"$e.png\" alt=\"$e\" align=\"middle\">";
}

$NATURAL = '<img src="natural.png">';  # without align

$dim = param('dim');
unless ($dim) {
    $dim = 3;
    param('dim', $dim);
}

print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Pseudo-separable L# Convex'),
    a({href=>"./"},'DCP HOME'), ln,
    h1('Pseudo-separable L<img src="natural.png" width=14> Convex Minimization'), ln,
    '<BLOCKQUOTE>', ln;


sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( length("$e")>0 and $e =~ m/([-+]?\d+)/
	 and $min <= int($1) and int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i, $j);

    for ($i=0; $i<$dim; $i++) {
	for ($j=0; $j<$i; $j++) {
	    set_param("aa:$i:$j", 0, 50);
	    set_param("bb:$i:$j", -50, 50);
	}
	set_param("a:$i", 1, 50);
	set_param("b:$i", -50, 50);
	set_param("c:$i", -50, 50);
	set_param("x:$i", -50, 50);
    }
}

sub print_html_table {
    init_param();
    print <<EOF;
f(x) = sum(aa[i][j] * (x[i]-x[j])^2 + bb[i][j] * (x[i]-x[j]), i&lt;j)
<br> &nbsp; &nbsp; &nbsp; + sum(a[i] * (x[i]-c[i])^2 + b[i] * (x[i]-c[i]), i)

<p> This f(x) function is L$NATURAL-convex [1,2].
This web application minimizes f(x) using
<a href="http://www.misojiro.t.u-tokyo.ac.jp/~tutimura/odicon/">ODICON</a>.
EOF

    print start_form, hidden('dim'), start_div({-align=>'center'}), ln;

    @index = (1..10);
    print p, 'dimension = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change'), ln;

    foreach $e ('aa', 'bb') {
	print br, start_table({-border=>'1'}), ln;
	print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ $e, 1 .. $dim ])]);
	for ($i=0; $i<$dim; $i++) {
	    my @td = ($i+1);
	    for ($j=0; $j<$i; $j++) {
		push @td, textfield(-name=>"$e:$i:$j", -size=>3, -default=>param("$e:$i:$j"));
	    }
	    print Tr({-align=>'right'}, [td([@td])]), ln;
	}
	print end_table, ln;
    }

    print br, start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ '', 1 .. $dim ])]);
    foreach $e ('a', 'b', 'c', 'x') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }
    print end_table, ln;

    print p, "(Note: An inappropriate value will be replaced with a random number.)\n";
    print p, submit('Minimize'), reset('Reset'), ln;
    print end_div, end_form, ln;
}

sub print_reference {
    print <<EOF;
<hr width="70%">

<p> [1] K. Murota, "Discrete Convex Analysis---An Introduction (in Japanese),"
Kyoritsu Publishing Company, Tokyo, (2001).

<p> [2] K. Murota,
"Discrete Convex Analysis," SIAM, (2003). 

<p> [3] K. Murota,
Recent developments in discrete convex analysis,
in: W. Cook, L. Lovasz and J. Vygen, eds.,
Research Trends in Combinatorial Optimization, Bonn 2008,
Springer-Verlag, Berlin, (2009), Chapter 11, 219--260.
EOF
}

sub change_dim {
    $dim = param('newdim');
    param('dim', $dim);
}

sub call_executable {
    print '<hr width="70%">', ln;
    print '<pre>', ln;
    open(OUT, "|./odicon DL $dim") || die('Can not call ./odicon');
    #open(OUT, '|cat') || die('Can not call ./odicon');
    foreach $e ('aa', 'bb') {
	for ($i=1; $i<$dim; $i++) { # we ignore [0][0]
	    for ($j=0; $j<$i; $j++) {
		print OUT param("$e:$i:$j"), ' ';
	    }
	    print OUT ln;
	}
    }
    foreach $e ('a', 'b', 'c', 'x') {
	for ($j=0; $j<$dim; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }
    close OUT;
    print '</pre>', ln;
}


if (param('Minimize')) {
    print_html_table();
    call_executable();
    print_reference();
} elsif (param('Change')) {
    change_dim();
    print_html_table();
    print_reference();
} else {
    print_html_table();
    print_reference();
}

print '</BLOCKQUOTE>', end_html, ln;
