/* minimization of M(natural)-convex function
 *  This is an implementation of greedy algorithm
 *  (improved steepest descent method).
 *  The calculation cost is O(dimension) per step,
 *  cutting area more strictly.
 *  cf. Akiyoshi SHIOURA, "Fast Scaling Algorithms for M-convex Fucntion
 *  Minimization with Application to the Resource Allocation Problem",
 *  2002.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "odicon.h"

/* returns index (for plus) if improvment was found. */
/* returns 'minus' if no improvment was found. */
static int best_improve(int dim, double func(int,int*), int x[],
                        int minus, int up[], int natural) {
    int i, min_i = minus;
    double f, min_f;

    iteration++;
    min_f = func(dim, x);  oracle_call++;
    x[minus]--;

    for (i=0; i<dim+natural; i++) {
        if (i == minus) continue;
        x[i]++;
        if (i == dim || x[i] <= up[i]) {
            f = func(dim, x);  oracle_call++;
            if (f < min_f) {  min_f = f;  min_i = i;  improve_call++;  }
        }
        x[i]--;
    }

    x[minus]++;
    return min_i;
}


/* returns the minimum value of func(x)    */
/* INPUT  func   : pointer to the function */
/* INPUT  init[] : the initial solution    */
/* OUTPUT init[] : the minimized solution  */
static double minimize(int dim, double func(int,int*), int init[],
                       int lower[], int upper[], int natural) {
    int *x, *lo, plus, minus, improved;

    counter_reset();
    if (!is_feasible(dim, func, init)) return PLUS_INFINITY;
    status_print(dim, func, init);

    assert(natural == 0 || natural == 1);
    x  = ivector_new(dim + natural);
    lo = ivector_new(dim + natural);
    ivector_copy(dim, x,  init);
    ivector_copy(dim, lo, lower);

    do {
        improved = FALSE;
        for (minus=0; minus<dim+natural; minus++) {
            if (minus == dim || lo[minus] < x[minus]) {
                plus = best_improve(dim, func, x, minus, upper, natural);
                if (plus != minus) {
                    improved = TRUE;
                    x[plus]++;
                    x[minus]--;
                }
                lo[plus] = x[plus];
            }
        }
    } while (improved);
    status_print(dim, func, x);
    ivector_copy(dim, init, x);
    free(x);
    free(lo);

    counter_print();
    return func(dim, init);
}


double mconv_minimize3(int dim, double func(int,int*), int init[],
                       int lower[], int upper[]) {
    return minimize(dim, func, init, lower, upper, 0);
}

double mgconv_minimize3(int dim, double func(int,int*), int init[],
                       int lower[], int upper[]) {
    return minimize(dim, func, init, lower, upper, 1);
}
